Package: NewmanOmics
Type: Package
LazyData: true
Title: Extending the Newman Studentized Range Statistic to
        Transcriptomics
Version: 1.1.3
Date: 2026-06-06
Authors@R: c(person(given = "Zachary B.", family = "Abrams", role = "aut"),
               person(given = "Greg", family = "Gershkowitz", role = "aut"),
               person(given = "Anoushka", family = "Joglekar", role = "aut"),
               person(given = "Chao", family = "Liu", role = "aut"),
               person(given = c("Kevin", "R."), family = "Coombes",
                      role = c("aut", "cre"), email = "krc@silicovore.com"))
Description: Extends the classical Newman studentized range statistic
  in various ways that can be applied to genome-scale transcriptomic
  or other expression data.
License: Apache License (== 2.0)
Depends: R (>= 4.4)
Imports: methods, stats, graphics, grDevices, oompaBase
VignetteBuilder: knitr
Suggests: knitr, rmarkdown
URL: http://oompa.r-forge.r-project.org/
NeedsCompilation: no
Packaged: 2026-07-06 13:44:07 UTC; kevin
Author: Zachary B. Abrams [aut],
  Greg Gershkowitz [aut],
  Anoushka Joglekar [aut],
  Chao Liu [aut],
  Kevin R. Coombes [aut, cre]
Maintainer: Kevin R. Coombes <krc@silicovore.com>
Repository: CRAN
Date/Publication: 2026-07-06 14:20:19 UTC
Built: R 4.6.1; ; 2026-07-06 14:32:57 UTC; unix

import("methods")

importFrom("stats", "approxfun", "density", "integrate", "loess",
           "pnorm", "predict", "rnorm", "sd",
           "dbeta", "nlm", "optimize")
importFrom("graphics", "plot", "hist", "points", "legend",
           "lines", "mtext", "par")
importFrom("grDevices", "dev.interactive", "devAskNewPage")

importFrom("oompaBase", "matrixMean", "matrixVar")

export("bankStat", "createBank")
export("pairedStat")

exportMethods("plot", "hist", "[", "dim")

exportClass("MixOf3Beta")
export("fitMix3", "computeFDR", "computeCutoff")
exportMethods("image")

library(NewmanOmics)

data(LungPair)
lung <- as.matrix(log2(1 + LungPair))
summary(lung)

set.seed(12345)
normal <- lung[, 1, drop=FALSE]
tumor  <- lung[, 2, drop=FALSE]

ps <- pairedStat(normal, tumor)
slotNames(ps) # should be five
dim(ps@nu.statistics)
dim(ps@p.values)

summary(ps@nu.statistics)
summary(ps@p.values)

head(ps@nu.statistics)
head(ps@p.values)

ps2 <- pairedStat(list(lung))
summary(ps2@nu.statistics)
summary(ps2@p.values)

summary(ps@nu.statistics - ps2@nu.statistics)
summary(pdiff <- ps@p.values - ps2@p.values)

plot(ps@nu.statistics, pdiff)
abline(h=0)

plot(ps)
hist(ps)

library(NewmanOmics)
## simulate a data set
set.seed(86420)
NR <- 5000
NC <- 20
N2 <- matrix(rnorm(NR*NC), nrow = NR, ncol = NC)
C2 <- matrix(rnorm(NR*NC), nrow = NR, ncol = NC)
## Compute stats theoretically
T1 <- pairedStat(N2, C2, ptype = "theo", ntype = "one")
T2 <- pairedStat(N2 ,C2, ptype = "theo", ntype = "two")
# compute stats empirically
E1 <- pairedStat(N2, C2, ptype = "emp", ntype = "one")
E2 <- pairedStat(N2, C2, ptype = "emp", ntype = "two")
## test agreement of nu.statistics
all(T1@nu.statistics > 0) # absolute value already computed
all(T2@nu.statistics == E2@nu.statistics) # signed values agree across methods
all(T1@nu.statistics == abs(T2@nu.statistics)) # theoretical agrees
all(E1@nu.statistics == abs(E2@nu.statistics)) # empirical agrees
## test agreement of p-values
fold <- function(p) { 1 - abs(1 - 2*p) } # convert two-sided to one-sided
all(T1@p.values == fold(T2@p.values))
range(E1@p.values - fold(E2@p.values)) # variability from simulations
range(T1@p.values - E1@p.values)       # variability from simulations
range(T2@p.values - E2@p.values)       # variability from simulations

setClass("MixOf3Beta",
         slots=c(
           input = "numeric", # input data vector
           mle = "numeric",   # parameters: L and M
           psi = "numeric",   # proportions: alpha, beta, gamma
           Z = "matrix"))     # latent indicator variable

validBetaMix <- function(object) {
  (all(object@input >= 0) & all(object@input <= 1) &
   length(object@mle) == 2 &
   length(object@psi) == 3 &
   ncol(object@Z) == 3 &
   nrow(object@Z) == length(object@input))
}
setValidity("MixOf3Beta", validBetaMix)

setMethod("plot", signature = c("MixOf3Beta", "missing"), function(x, y, ...) {
  who <- apply(x@Z, 1, function(w) {sample(1:3, 1, prob=w)})
  if (all(par("mfrow") == 1)) {
    opar <- par(mfrow=c(3,1))
    on.exit(par(opar))
  }
  bk <- seq(0, 1, length = 57)
  hist(x@input[who==1], breaks=bk, main="Right", xlab="P")
  hist(x@input[who==2], breaks=bk, main="Left", xlab="P")
  hist(x@input[who==3], breaks=bk, main="Uniform", xlab="P")
  invisible(x)
})

setMethod("hist", signature = "MixOf3Beta", function(x, lcol="red", breaks=101, ...) {
  tt <- seq(0, 1, length=501)
  dd <- dMix3(x, tt)
  hist(x@input, breaks=breaks, prob=TRUE, ...)
  lines(tt, dd, lwd=2, col=lcol)
})

setMethod("image", signature = "MixOf3Beta", function(x, ...) {
  image(1:nrow(x@Z), 1:3, x@Z[order(x@Z[,1], x@Z[,2]),],
        xlab='', ylab='', yaxt='n', ...)
  mtext(c("Right", "Left", "Uniform"), side=2, at=1:3, las=2, line=1)
})

### Assumes a mixture of three beta distributions
###      alpha*Beta(L,1) + beta*Beta(1, M) + gamma*Beta(1,1)
###       right-peak        left-peak         uniform

### Note that dbeta(x; L, 1) = L*x^(L-1)
###       and dbeta(x; 1, M) = M*(1-x)^(M-1)
###       and dbeta(x; 1, 1) = 1, is uniform on the interval [0,1].

## likelihood = P(q | L, M, z)
##     params = c(L, M)
##     q = observed data vector
##     z = latent indicator matrix
logLikelihood <- function(param, q, z) {
  L <- param[1]
  M <- param[2]
  N <- length(q)
  like <- z[,1]*L*q^(L-1) + z[,2]*M*(1-q)^(M-1) + z[,3]
  -sum(log(like)) # negate because 'nlm' finds minima instead of maxima
}

## datavec is the vector of observed data, assumed to be between 0 and 1
fitMix3 <- function(datavec, forever=100,
                    epsilon=0.001, relative = 0.001, print.level=0) {
  if (any(is.na(datavec))) {
    warning("Data contains missing values; we are ignoring them.")
    datavec <- datavec[!is.na(datavec)]
  }
  if (any(datavec > 1) | any (datavec < 0)) {
    stop("Data values must be in the inteval [0,1].")
  }
  ## initialize cluster assignments
  N <- length(datavec)
  Z <- matrix(0, nrow=N, ncol=3) # binary cluster assignment indicators
  temp <- data.frame(dbeta(datavec, 8, 1), # right peak beta
                     dbeta(datavec, 1, 8), # left peak beta
                     dbeta(datavec, 1, 1)) # uniform
  who <- apply(temp, 1, function(x) {sample(1:3, 1, prob=x)})
  Z[who == 1, 1] <- 1
  Z[who == 2, 2] <- 1
  Z[who == 3, 3] <- 1
  ## loop control
  lastlike <- -10^5
  currlike <- 0
  delta <- abs(currlike - lastlike)
  reld <- 2*relative
  count <- 0
  mle <- c(2,2)
  while ((count < forever) & (delta > epsilon) & (reld > relative)) {
    count = count + 1
    ## M-step
    maxlike <- nlm(logLikelihood, mle, q=datavec, z=Z, stepmax=10000,
                   print.level=print.level)
    mle <- maxlike$estimate
    lastlike <- currlike
    currlike <- maxlike$minimum
    if (any(mle < 0)) stop("One of the mle's is negative!")
    ## E-step
    psi <- apply(Z, 2, mean) # tentative proportions of mixture components
    Z[,1] <- psi[1]*dbeta(datavec, mle[1], 1)
    Z[,2] <- psi[2]*dbeta(datavec, 1, mle[2])
    Z[,3] <- psi[3]
    Z <- sweep(Z, 1, apply(Z, 1, sum), "/")
    if (print.level > 0) {
      cat("currliike = ", currlike, " delta = ", delta, "\n")
      cat("MLE (L,M) = ")
      cat(mle, "\n")
      cat("\nPsi (p1, p2, p3) = ")
      cat(psi, "\n")
    }
    delta <- abs(lastlike - currlike)
    reld <- abs(delta/currlike)
  }
  new("MixOf3Beta",
      input = datavec,
      mle = mle,
      psi = psi,
      Z = Z)
}

# the density function for the fitted distribution
dMix3 <- function(object, x) {
  psi <- object@psi
  L <- object@mle[1]
  M <- object@mle[2]
  psi[1]*L*x^(L-1) + psi[2]*M*(1-x)^(M-1) + psi[3]
}

# the cumulative probability function for the fitted distribution
# should be
#          alpha * x^L  +  beta * (1 - (1 - x)^M)  +  gamma * x
pMix3 <- function(object, x) {
  psi <- object@psi
  L <- object@mle[1]
  M <- object@mle[2]
  psi[1]*x^(L) + psi[2]*(1 - (1-x)^M) + psi[3]*x
}

# compute FDR given cutoff on nominal p-value
# note consistency with above description of CDF
computeFDR <- function(object, alpha) {
  mle <- object@mle
  psi <- object@psi
  A <- psi[2] * (1 - (1 - alpha)^mle[2])
  B <- psi[1] * alpha^mle[1] + psi[3] * alpha
  B/(A+B)
}

# compute cutoff on nominal p-value given desired FDR
computeCutoff <- function(object, fdr) {
  f <- function(x) abs(computeFDR(object, x) - fdr)
  optimize(f, c(0,1))
}

# BANKED

roam <- function(M) {
  as.vector(matrixMean(M))
}

stand <- function(M, mu) {
  sqrt(as.vector(matrixVar(M, mu)))
}

### INTERNAL
### Get the background distribution of "unusual" values in the bank
### KRC: why are we going to all the exta trouble of recomputing the loess
###      by removing each individual sample. Does that even matter?
nuValsIterBank <- function(matrix) {
  n <- nrow(matrix)
  s <- ncol(matrix)
  temp.means = temp.sd = matsdEst = matNu = matrix(NA,n,s)

  for (j in 1:s){
    temp.means[ ,j] <- roam(matrix[ ,-j])
    temp.sd[ ,j] <- stand(matrix[ ,-j], temp.means[,j])
    l.mod <- loess(temp.sd[ ,j] ~ temp.means[ ,j])
    matsdEst[ ,j] <- predict(l.mod)
  }
  matNu <- (matrix - temp.means)/matsdEst
  return(matNu)
}

### EXTERNAL
### prepare a "bank" of normal samples against which to test an individual
createBank <- function(bankMatrix) {
  ## warn if data appears to be on the raw scale
  if (diff(range(bankMatrix)) > 100) {
    warning("createBank: Data appears to be on the raw scale and not on a log scale.")
  }
  ## Compute row-wise mean and standard deviation
  bankMeans  <- roam(bankMatrix)
  bankSD <- stand(bankMatrix, bankMeans)
  ## Fit a loess model, since we assume SD is a smooth function of the mean
  loess.model <- loess(bankSD ~ bankMeans)
  sdEst <- predict(loess.model)
  ## bundle the results
  rowStats <- data.frame(mean=bankMeans, SD=bankSD, sdEst=sdEst)
  ##
  background <- nuValsIterBank(bankMatrix)
  ## return a list
  list(rowStats = rowStats, background = background)
}

### EXTERNAL
### Primary interface; compute the actual Newman banked statistic
bankStat <- function(bankObj, testSet, bankMatrix){
  n <- dim(testSet)[1]
  s <- dim(testSet)[2]

  if(missing(bankObj)){
    bankObj <- createBank(bankMatrix)
  } else if (!missing(bankMatrix)) {
    stop("You should only supply one of 'bankObj' or 'bankMatrix'.")
  }
  temp.means <- bankObj$rowStats$mean
  sdEst <- bankObj$rowStats$sdEst
  Back_dist_iter <- bankObj$background

  matNuBank <- nuValsBank(matrix = testSet, mean = temp.means, sd = sdEst)

  matPBanked <- pValuesBanked(matrix = matNuBank, Background = Back_dist_iter, n, s)

  return(list(nu.statistics = matNuBank, p.values = matPBanked))

}
### If an input is missing, for example the bankObj, run your code as
###       pvals_uncorr = bankStat(,testSet,bankMatrix)

nuValsBank<- function(matrix, mean, sd){
  centered <- sweep(matrix, 1, mean, "-")
  scaled <- sweep(centered, 1, sd, '/')
  return(scaled)
}

## Looks stupid slow. Can we speed it up?
pValuesBankedOrig <- function(matrix, Background, n, s){
  meanDist = mean(Background)
  sdDist = sd(Background)

  matP = matrix(NA,n,s)
  for (i in 1:n){
    for (j in 1:s){
      matP[i,j] <- pnorm(matrix[i,j],meanDist,sdDist)
    }
  }
  return(matP)
}
pValuesBankedSecond <- function(matrix, Background, n, s){
  meanDist = mean(Background)
  sdDist = sd(Background)
  pnorm(matrix, meanDist, sdDist)
}
pValuesBanked <- function(matrix, Background, n, s){
  pnorm(matrix, 0, 1)
}

## If pvals_uncorr is your output, fdr gives you a matrix of corrected p values

# v <- as.vector(pvals_uncorr)
#
# BH_vec = p.adjust(v, method = 'fdr')
# fdr =  as.matrix(BH_vec)
# dim(fdr) = c(n,s)


### The following four functions were written in an effort to find p values efficiently. Even though they work for smaller sets, the run time is significantly less for the built in FDR function ###
altPValues <- function(matrix,Background,n,s){
  matP <- matrix(NA,n,s)
  v = as.vector(Background)
  vec = v[!duplicated(v)]

  for (i in 1:n){
    for (j in 1:s){
      matP[i,j] <- length(vec[vec <= matrix[i,j]])/length(vec)
    }
  }
  return(matP)
}

empiricalP <- function(matrix,background,n,s){
  pdf = approxfun(density(background))
  min = min(background)

  pf = function(q){integrate(pdf,min,q,subdivisions = 2000)$value}


  empirMatP = matrix(NA,n,s)

  for (i in 1:n){
    for (j in 1:s){
      if (matrix[i,j] >= min){empirMatP[i,j] = pf(matrix[i,j])}
      else {empirMatP[i,j] = 0.04}
    }
  }
  return(empirMatP)
}

fdrCorrection <- function(matrix, n, s, verbose = FALSE){

  ## Benjamini Hochberg method employed. Could also use built in p.adjust(p, "BH")
  ## Percent correction only factors in during analysis, b/c (rank/total)*percentageFDR only gives the adjusted cutoff


  v <- as.vector(matrix)
  vUnique <- v[!duplicated(v)]
  vs <- sort(vUnique,decreasing = T)
  vRank <- rank(vs)
  num <- max(vRank)

  adjustedP <- length(vUnique)
  adjustedP[1] <- vs[1]

  ##### if statement

  # breakPt = floor(0.9*num)
  # for (i in 1:breakPt){adjustedP[i] = 0.499}
  #
  # for (i in breakPt+1:num){
  #   adjustedP[i] <- min(adjustedP[i-1],(vs[i]*num)/vRank[i])
  #   if(i%%((num-breakPt)/10) == 0)print((i/(num-breakPt))*100)
  # }
  #

  for (i in 1:num){
    adjustedP[i] <- min(adjustedP[i-1],(vs[i]*num)/vRank[i])
    if(verbose & i %% (num/10) == 0) print((i/num)*100)
  }

  fdrP <- matrix(NA,n,s)
  for (i in 1:n){
    for (j in 1:s){
      fdrP[i,j] <- adjustedP[match(matrix[i,j],vUnique)]
    }
  }
  return(fdrP)
}

fdrCutoff <- function(matrix,alpha){

  v <- as.vector(matrix)
  w <- sort(v)
  threshold <- alpha*(1:length(v))/length(v)
  c <- cbind(v,threshold,v<=threshold)

  sig_value <- c[,1][c[,3] == 1]
  cutoff <- max(sig_value)

  return(cutoff)


  ## If we want to input a cutoff then use this method, else use FDR correction on p-values
  # sigP <- fdrCutoff(matPBanked,alpha = 0.05)
  # binarySigBank <- matrix(0,n,s)
  #
  # for (i in 1:n){
  #   for (j in 1:s){
  #     if (matPBanked[i,j] <= sigP){binarySigBank[i,j] = 1}
  #   }
  #   if(i%%(n/10) == 0)print((i/n)*100)
  # }
}
#####################

setClass("NewmanPaired",
         slots = c(
           nu.statistics = "matrix",
           p.values = "matrix",
           pairedMean = "matrix",
           difference = "matrix",
           smoothSD = "matrix")
)

validNewmanPair <- function(object) {
  all((dim(object@nu.statistics) == dim(object@p.values))
      & (dim(object@nu.statistics) == dim(object@pairedMean))
      & (dim(object@nu.statistics) == dim(object@difference))
      & (dim(object@nu.statistics) == dim(object@smoothSD)))
}
setValidity("NewmanPaired", validNewmanPair)

setMethod("[", signature = "NewmanPaired", function(x, i, j, ..., drop=FALSE) {
  new("NewmanPaired",
      nu.statistics = x@nu.statistics[i,j, drop=FALSE],
      p.values = x@p.values[i,j, drop=FALSE],
      pairedMean = x@pairedMean[i,j, drop=FALSE],
      difference = x@difference[i,j, drop=FALSE],
      smoothSD = x@smoothSD[i,j, drop=FALSE])
})

setMethod("dim", signature = "NewmanPaired", function(x) {
  dim(x@nu.statistics)
})

setMethod("plot", signature = c("NewmanPaired", "missing"),
          function(x, y, which = NULL, ask = NULL,
                   high=0.99, low=0.01, colset=c("red", "blue", "orange"), ...) {
  M <- dim(x)[2]
  if (is.null(which)) {
    which <- 1:M
  }
  if (any(which < 1) || any(which > M)) {
    stop("'which' must be between", 1, "and", M, "\n")
  }
  if (is.null(ask)) {
    ask <- prod(par("mfcol")) < length(which) && dev.interactive()
  }
  if (ask) {
    oask <- devAskNewPage(TRUE)
    on.exit(devAskNewPage(oask))
  }
  for (W in which) {
    X <- x[,W]
    bigp <- X@p.values > 0.99
    smallp <- X@p.values < 0.01
    plot(X@pairedMean, X@difference,
         main=colnames(X@pairedMean),
         xlab="Mean log expression", ylab="Difference in log expression")
    points(X@pairedMean[smallp], X@difference[smallp], col=colset[1], pch=16)
    points(X@pairedMean[bigp], X@difference[bigp], col=colset[2], pch=16)
    points(X@pairedMean, X@smoothSD, col=colset[3])
    points(X@pairedMean, -X@smoothSD, col=colset[3])
    legend("topleft",
           c(paste("P <", round(low, 3)),
             paste("P >", round(high, 3)),
             "Smoothed SD"),
           col=colset, pch=16)
  }
  invisible(x)
})

setMethod("hist", signature = "NewmanPaired",
          function(x, breaks=101, which=NULL, ask=NULL, xlab="P-value", ...) {
  M <- dim(x)[2]
  if (is.null(which)) {
    which <- 1:M
  }
  if (any(which < 1) || any(which > M)) {
    stop("'which' must be between", 1, "and", M, "\n")
  }
  if (is.null(ask)) {
    ask <- prod(par("mfcol")) < length(which) && dev.interactive()
  }
  if (ask) {
    oask <- devAskNewPage(TRUE)
    on.exit(devAskNewPage(oask))
  }
  for (W in which) {
    X <- x[,W]
    hist(X@p.values, breaks=breaks, xlab=xlab,
         main=colnames(X@p.values), ...)
  }
})

pairedStat <- function(baseData, perturbedData = NULL, pairing = NULL,
                       ptype = c("empirical", "theoretical"),
                       ntype = c("one-sided", "two-sided")){
  ptype <- match.arg(ptype)
  ntype <- match.arg(ntype)
  if (is.list(baseData)) {
    x <- baseData
    baseData <- do.call(cbind, lapply(x, function(entry) {entry[,1]}))
    colnames(baseData) <- sapply(x, function(A) colnames(A)[1])
    perturbedData <- do.call(cbind, lapply(x, function(entry) {entry[,2]}))
    colnames(perturbedData) <- sapply(x, function(A) colnames(A)[2])
    rm(x)
  } else if (is.null(perturbedData)) {
    if (is.null(pairing)) {
      stop("You must supply at least one of 'perturbedData' or 'pairing'.")
    }
    pos <- which(pairing > 0)
    pos <- pos[order(pairing[pos])]
    neg <- which(pairing < 0)
    neg <- neg[order(abs(pairing[neg]))]  
    x <- baseData
    baseData <- x[,neg,drop=FALSE] # 'drop' in case there is only one pair
    perturbedData <- x[,pos, drop=FALSE]
    rm(x, pos, neg)
  }
  
  ## KRC: Do we need to check that the two matrices are the same size?
  ## Or just let the first computation throw its own error?

  ## Matrix computation of mean of two things
  pairedMean <- (baseData + perturbedData) / 2
  ## Similar computation for SD of two things.
  pooledSD <- abs(baseData - perturbedData) / sqrt(2)
  colnames(pairedMean) <- colnames(pooledSD) <- colnames(perturbedData)
  ## For each column, perform loess fit
  n <- dim(baseData)[1]
  s <- dim(baseData)[2]
  smoothSD <- matrix(NA, n, s) # set aside storage
  for (i in 1:s) {
    l.mod <- loess(pooledSD[ ,i] ~ pairedMean[ ,i])
    smoothSD[ ,i] <- predict(l.mod)
  }
  colnames(smoothSD) <- colnames(pairedMean)
  
  ## compute the matrix of nu-statistics
  ## always compute the signed (two-sided) version as default
  matNu <- (baseData - perturbedData) / smoothSD
  colnames(matNu) <- colnames(pairedMean)

  ## empirical p-values via simulation
  emp <- function(M, ntype) {
     m <- mean(M)
     sd <- sd(abs(M))
     randNu <- randNuGen(m, sd)
     pvp <- nu2PValPaired(M, as.vector(randNu))
     colnames(pvp) <- colnames(pairedMean)
     1 - pvp
  }
  theo <- function(M, ntype) {
    pnorm(M, 0, sqrt(pi))
  }
  pValsPaired <- switch(ptype,
                        empirical = emp(matNu),
                        theoretical = theo(matNu)
                        )
  if (ntype == "one-sided") {
    matNu <- abs(matNu)
    pValsPaired <- 1 - abs(1 - 2*pValsPaired)
  }

  new("NewmanPaired",
      nu.statistics = matNu,
      p.values = pValsPaired,
      pairedMean = pairedMean,
      difference = perturbedData - baseData,
      smoothSD = smoothSD)
}

### Generating 1 million Nu values based on the overall mean and std deviation
### of the Nu values obtained from the paired statistic. This will later be
### used to estimate the p-values.
randNuGen <- function(mu=0, sigma=1) {
  ## magic numbers:  ngenes = 10000, ntimes = 100
  A <- matrix(rnorm(10000*100, mu, sigma), ncol=100)
  B <- matrix(rnorm(10000*100, mu, sigma), ncol=100)
  sdest <- mean( abs(A-B)/sqrt(2) )
  (A-B)/sdest
}

### originally written by Chao Liu on stackoverflow at
### https://stackoverflow.com/questions/20133344/find-closest-value-in-a-vector-with-binary-search
NearestValueSearch <- function(x, w){
  ## A simple binary search algorithm
  ## Assume the w vector is sorted so we can use binary search
  left <- 1
  right <- length(w)
  while(right - left > 1){
    middle <- floor((left + right) / 2)
    if(x < w[middle]){
      right <- middle
    }
    else{
      left <- middle
    }
  }
  if(abs(x - w[right]) < abs(x - w[left])){
    return(right)
  }
  else{
    return(left)
  }
}

nu2PValPaired <- function(nuMatrix, vec){
  vec <- sort(vec)
  MatP <- matrix(sapply(nuMatrix, function(x) {
    1 - NearestValueSearch(x, vec)/length(vec)
  }), nrow(nuMatrix), ncol(nuMatrix))
  return(MatP)
}

---
title: "NewmanOmics: Tools for Personalized Transcriptomics"
author: "Kevin R. Coombes"
data: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{NewmanOmics}
  %\VignetteKeywords{NewmanOmics,Personalized Transcriptomics,Paired Test,Banked Test}
  %\VignetteDepends{NewmanOmics}
  %\VignettePackage{NewmanOmics}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include=FALSE, results="hide"}
knitr::opts_chunk$set(echo = TRUE, fig.width=7,fig.height=5,echo=TRUE)
```

In this vignette, we describe how to use the NewmanOmics Paired and
Banked tests to analyze gene expression data from a single sample.

# Getting Started

As usual, we start by loading the package:

```{r libs}
library(NewmanOmics)
```

The package contains paired tumor and normal samples from patients with
head and neck cancer. these came from a study that was submitted to the
Gene Expression Omnibus.
```{r GSE6631}
data(GSE6631)
dim(GSE6631)
GSE6631[1:5, 1:4]
```
As we can see, this consists of (normalized) Affymetrix microarray data.
The odd numbered columns are derived from normal mucosa, and the even numbered
columns are derived from paired tumor samples.

Before proceeding, we are going to log-transform the data.
```{r HN, fig.cap="Box plot of log-transformed data."}
HN <- log2(1 + GSE6631)
boxplot(HN, col=c("forestgreen", "dodgerblue"))
```
The figure suggests that the the data have been reasonably normalized, and that
it is unlikely to be overwhelmed by artifacts.

# Paired Statistic
To illustrate the Newman Paired test, we are going to use only one sample.
```{r HN1}
HN1 <- HN[, 1:2]
result1 <- pairedStat(HN1, pairing = c(-1,1))
summary(result1@nu.statistics)
summary(result1@p.values)
```
We can create a histogram of the per-gene (empirical) p-values
```{r fig.cap="Histogram of empoirical p-values."}
hist(result1)
```
We can also produce an "M-versus-A" plot of the data.
```{r fig.cap="Bland-Altman plot."}
plot(result1)
```

## Alternate Inputs
The <tt>pairedStat</tt> function has flexible inputs, allowing you to store the data 
in various ways. Here we run the algorithm for three pairs, with an explicit pairing vector.
```{r}
result2 <- pairedStat(HN[, 1:6], pairing=c(-1, 1, -2, 2, -3, 3))
summary(result2@nu.statistics)
summary(result2@p.values)
```

```{r fig.cap="Bland-ALtman plots."}
plot(result2)
```

```{r fig.cap="P-value histograms."}
hist(result2)
```

We can also input the same data as a pair of matrices.
```{r HN3}
normals <- HN[, c(1,3,5)]
tumors <- HN[, c(2,4,6)]
result3 <- pairedStat(normals, tumors)
summary(result3@nu.statistics)
summary(result3@p.values)
```

Or we can input the same data as a list of paired samples.
```{r HN4}
listOfPairs <- list(HN[,1:2], HN[,3:4], HN[,5:6])
result4 <- pairedStat(listOfPairs)
summary(result4@nu.statistics)
summary(result4@p.values)
```

# Banked Statistic
A completely different approach to personalized transcriptomics is to compare
individual samples to a "bank" of known normals.

```{r makeBank}
normals <- HN[, seq(1, ncol(HN), 2)] # odds are normal
tumors <- HN[, seq(2, ncol(HN), 2)] # evens are tumor
bank <- createBank(normals)
result5 <- bankStat(bank, tumors[,1,drop=FALSE])
summary(result5$nu.statistics)
summary(result5$p.values)
hist(result5$p.values, breaks=101)
```

\name{MixOf3Beta-class}
\alias{MixOf3Beta}
\alias{MixOf3Beta-class}
\alias{plot,MixOf3Beta,missing-method}
\alias{hist,MixOf3Beta-method}
\alias{image,MixOf3Beta-method}
\docType{class}
\title{Class "MixOf3Beta"}
\description{
  Represents the results of fitting a beta-mixture model to a set of
  p-values that has peaks at both zero and one.
}
\details{
  Given a set of p-values (or any data on the interval [0,1]) that has
  peaks at both ends of the interval, we fit a three-componet mixture
  model. One component is uniform, and represents the expected
  distribution under the null hypothesis that nothing interesting is
  happening anywhere. The second component has the distribution
  Beta(1,M); this has a peak at zero and represents the features of
  interest. The final component has the distribution Beta(L,1). In the
  context of the Newman paired statistic, this represents genes or
  features whose variabilirt is smaller than the locally smoothed
  estimate of the standard deviation; we can think of these as
  "extra boring".
}
\section{Creating Objects}{
  In practice, users will use the \code{\link{fitMix3}} function to
  construct an object of the \code{MixOf3Beta} class. Hand
  construction is strongly discouraged.
}
\section{Slots}{
  \describe{
    \item{\code{input}:}{A numeric vector containing the input p-values.}
    \item{\code{mle}:}{ A numeric vactor of length 2 containing the beta
      parameters \code{L} and \code{M} (in that order).}
    \item{\code{psi}:}{A numeric vector of length three containing the
      mixing parameters, in the order (right-peak component, left-peak
      component, and uniform-component).}
    \item{\code{Z}:}{A matrix of size N (number of features) by 3. This
      contains the latent indicator matrix. Each row corresponds to a
      gene or feature, and the entries show the proabbiltiy that the
      feature arose from the right, left, or uniform comnponent.}
  }
}
\section{Methods}{
  \describe{
    \item{plot(x, y, ...)}{Plot the decompositon of the data into thre pieces.}
    \item{hist(x, lcol = "red", breaks=101, ...)}{Plot a histogram of
      the p-values along with the fitted model of the distribution.}
    \item{image(x)}{Plot a (sorted) image of the latent variable Z-matrix.}
  }
}
\references{
  Abrams ZB, Joglekar A, Gershkowitz GR, Sinicropi-yao S, Asiaee A,
  Carbone DP, Coombes KR. Personalized Transcriptomics: Selecting Drugs
  Based on Gene Expression Profiles. Preprint.
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com}
}
\seealso{
  \code{\link{pairedStat}}, \code{\link{NewmanPaired-class}}
}
\examples{
set.seed(98765)
ds <- c(rbeta(3000, 20, 1),
        rbeta(1000, 1, 40),
        runif(6000))
fit <- fitMix3(ds)
image(fit, col=topo.colors(64))
hist(fit, col="skyblue", lcol="blue")
plot(fit)
}
\keyword{classes}
\keyword{htest}
\keyword{multivariate}

\name{pairedStat}
\alias{pairedStat}
\title{Paired Newman Statistic}
\description{
The Paired Newman Statistic is used for one-to-one comparison of
paired individual samples. Commonly used to find differential expression
between tumor-normal pairs or before-after treatment pairs.
}
\usage{
pairedStat(baseData, perturbedData = NULL, pairing = NULL,
           ptype = c("empirical", "theoretical"),
           ntype = c("one-sided", "two-sided"))
}
\arguments{
\item{baseData}{Either a list or a matrix. May contain data for just the
  base condition (for example, normal samples or samples before
  treatment) or for both the base condition and the perturbed condition
  (for example, tumor samples or samples after treatment). See details.}
\item{perturbedData}{An optional matrix containing data for the "perturbed"
  samples. May be NULL if the \code{baseData} argument is a list or a
  matrix containing all the data.}
\item{pairing}{An optional vector indicating the pairing between base
  and perturbed samples. Entries must be integers. Positive integers
  indicate perturbed samples and negative integers with the same
  absolute value indicate the paired base samples. See details.}
\item{ptype}{Enumerated character string indicating which method to use
  to compute p-values.}
\item{ntype}{Enumerated character string indicating where to return
  one-sided (all positive) or two-sided (signed) nu-statistics with
  corresponding p-values.}
}
\value{
An object of the \code{\link{NewmanPaired}} class.
}
\details{
  In the simplest case, we have gene expression data on one "base"
  sample and one "perturbed" sample, and the goal is to identify genes
  whose expression changes between the two states. Our primary
  assumption is that the standard deviation (SD) of gene expression
  varies as a smooth function of the mean; fitting such a curve allows
  us to detect individual genes whose difference is large compared to
  the smoothed SD.

  Note that this assumption is most useful on the log-transformed
  scale  (\url{https://pubmed.ncbi.nlm.nih.gov/25092958/}).
  If your data is on a raw scale, then we recommend transforming
  it before computing the Newman paired statistic.

  The input arguments to the \code{pairedStats} function are moderately
  complicated in order to allow users to choose a convenient method for
  supplying data when they have multiple paired samples. The first
  posssibility is to have all the base samples in one matrix and all the
  perturbed samples in a second matrix. In this case, we assume (without
  checking) that the columns in the two matrices correspond to the
  paired samples, and that the genes-rows are in the same order.

  The second possibility is to put the data for both the base samples
  and the perturbed samples in the same matrix. In this case, the user
  must supply a \code{pairing} vector to explain how the samples should
  be matched. If the column order is ("base1", "perturbed1", "base2",
  "perturbed2", ...), then the pairiing vector should be written as
  \code{c(-1, 1, -2, 2, -3, 3, ...)}.

  The third possibility is to provide the paired samples in a list,
  each of whose entries is a matrix with two columns,with the first
  column being the base state and the second column being the
  perturbed state.

  This flexibility means that there are three equivalent ways to input
  the data even if you have only one base sample (with data in the
  one-column matrix B) and one perturbed sample (with data in the
  one-column matrix P). If we let \code{BP <- cbind(B, P) }, then we can
  choose (1) \code{pairedStats(B, P)}, or (2)
  \code{pairedStats(list(BP))}, or (3)  \code{pairedStats(BP,
    pairing = c(-1,1))}.

  The final two option, \code{ptype} and \code{ntype}, have been added
  for backwards comatibility. The default values match the performance
  of the original versions of the package, which returned the absolute
  values of the nu-statistics and computed empirical p-values using null
  simulations. Over time, we came to realize that the signed
  nu-statistics were simetimes useful. We also belatedly relaized that
  we could compute theoretical p-values from a particular normal
  distribution, N(0, sqrt(pi)), which arises becaue the smoothing step
  is equivalent in the null case to a transformation built from the
  half-normal distribution.
}
\examples{
data(GSE6631)
Normal <- GSE6631[, c(1,3)]
Tumor <- GSE6631[, c(2,4)]

### input two separate matrices
ps0 <- pairedStat(Normal, Tumor)
summary(ps0@nu.statistics)
summary(ps0@p.values)

### use the theoreticl p-values
ps1 <- pairedStat(Normal, Tumor, ntype = "two-sided")
folded <- 1 - abs(1 - 2*ps1@p.values)
summary(as.vector(folded - (ps0@p.values)))

### input one combined matrix and a pairing vector
ps2 <- pairedStat(GSE6631, pairing=c(-1, 1, -2, 2))
summary(ps2@nu.statistics)
summary(ps2@p.values)

### input a list of matrix-pairs
ps3 <- pairedStat(list(One = GSE6631[, 1:2],
                       Two = GSE6631[, 3:4]))
summary(ps3@nu.statistics)
summary(ps3@p.values)
}

\name{fitMix3}
\alias{fitMix3}
\alias{computeFDR}
\alias{computeCutoff}
\title{Compute FDR from Three-Component Beta Mixture}
\description{
  Provides functions to fit a beta-mixture model to a set of p-values
  that has peaks at both zero and one, and to estimate false discovery
  rates.
}
\usage{
fitMix3(datavec, forever=100, epsilon=0.001, relative = 0.001, print.level=0)
computeFDR(object, alpha)
computeCutoff(object, fdr)
}
\details{
  We have observed empirically that the set of p-values obtained when
  computing the Newman paired test statistic often has peaks both at zero
  (representing genes of interest) and at one (representing "boring"
  genes that change much less than expected). We attribute the latter
  phenomenon to the fact that we use locally smoothed instead of
  gene-by-gene estimates of the standard deviation; genes whose SD is
  increased by the smoothing process contribute to the boring peak near
  one.

  To estimate p-values in this context, we fit a three-component beta
  mixture model, combining (1) a right-peaked distribution Beta(L,1),
  (2) a left-peaked dfistribution Beta(1,M), and (3) a uniform
  distribution. Specfically, we look for models of the form
  \deqn{alpha*Beta(L,1) + beta*Beta(1, M) + gamma*Beta(1,1)}.
  
  Model-fitting uses an expectation-maximization (EM) algorithm. In
  addition to the parameters \code{mle=c(L,M)} and \code{psi=c(alpha,
  beta, gamma)}, we introduce a matrix \code{Z} of latent variables that
  indicate which distribution each point is likely to arise
  form. \code{Z} has three columns (one for each mixture component) and
  one row for each p-value; the entries in each row are nonegative and
  sum to one. The M-step of the algorithm uses the \code{\link[stats]{nlm}}
  optimization function to compute the maximum-likelihood \code{mle}
  values given \code{psi} and \code{Z}. The E-step first updates
  \code{psi} from the \code{Z}-matrix, and then updates the values of
  \code{Z} based on the current \code{mle}. 
  
  We are able to use the mixture distribution to compute the
  relationship between a cutoff on the nominal p-values and the false
  discovery rate (FDR). 
}
\arguments{
\item{datavec}{A numeric vector containing p-values.}
\item{forever}{An integer; maximum number of iterations while fitting
  the mixture model.}
\item{epsilon}{A real number; change in the log likelihood that should
  be used to terminate the model-fitting loop.}
\item{relative}{A real number; change in the relative log likelihood
  that should be used to terminate the model-fitting loop.}
\item{print.level}{An integer; how much detail should \code{\link[stats]{nlm}}
  print while fitting the model.}
\item{object}{An object of the \code{\link{MixOf3Beta}} class.}
\item{alpha}{A real number between 0 and 1; the cutoff on the nominal
  p-value where the FDR should be computed.}
\item{fdr}{A real number beteen 0 and 1; the targeted FDR value.}
}
\value{
  The model-fitting function, \code{fitMix3}, returns an object of the
  \code{\link{MixOf3Beta}} class.

  The \code{computeFDR} function returns a real number in [0,1], the
  false discovery rate assiociated with the nominal cutoff.

  The \code{computeCutoff} function returns a real number in [0,1], the
  cutoff required to achieve the desired FDR.
}
\examples{
set.seed(98765)
ds <- c(rbeta(3000, 20, 1),
        rbeta(1000, 1, 40),
        runif(6000))
fit <- fitMix3(ds)
computeFDR(fit, 0.01)
computeCutoff(fit, 0.01)
computeFDR(fit, 0.0016438)
computeCutoff(fit, 0.05)
computeFDR(fit, 0.00702114)
}

\name{newman-data}
\alias{newman-data}
\alias{LungPair}
\alias{GSE6631}
\docType{data}
\title{Datasets to Illustrate the Newman Tests}
\description{
  These data sets contain paired normal and tumor samples usewd to
  ilustrate te Newman pairted test and the Newman bank test. 
}
\usage{
data(LungPair)
data(GSE6631)
}
\format{\code{LungPair} is a data matrix containing normalized second
  generation sequencing data from The Cancer Genome Atlas (TCGA), with
  20,531 row (genes) and 2 columns (samples). The first column contains
  data for the normal sample and the second column contains data for the
  tumor sample from the patient with barcode TCGA.38.4625.

  \code{GSE6631} is a data matrix containing normalized Affymetrix
  microarray data from paired head-and-neck cancer samples in the Gene
  Expression Omnibus set GSE6631. The matrix contains 200 rows (a rando
  subset of genes) and 44 columns (samples). The odd numbered colukmns
  are derived from normal mucosa; te even numbered columns are derived
  from paired tumor samples from the same patient.
}
\source{
  The full squamous cell lung cancer (LUSC) data from TCGA was
  downloaded from   \url{http://firebrowse.org/}, and the data for this
  pair were separated and saved as a binary R data file. The
  head-and-neck cancer data were downloaded from the Gene Expression
  Omnibus at 
  \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE6631}. A
  subset of 2000 genes was randomly selected before saving the binary R
  data file.
}
\references{
  Kuriakose MA, Chen WT, He ZM, Sikora AG et al. Selection and
  validation of differentially expressed genes in head and neck
  cancer. Cell Mol Life Sci 2004 Jun;61(11):1372-83. 
}
\examples{
data(LungPair)
data(GSE6631)
}
\keyword{datasets}

\name{NewmanPaired-class}
\alias{NewmanPaired}
\alias{NewmanPaired-class}
\alias{plot,NewmanPaired,missing-method}
\alias{hist,NewmanPaired-method}
\alias{dim,NewmanPaired-method}
\alias{[,NewmanPaired-method}%] to make emacs more-or-less happy
\docType{class}
\title{Class "NewmanPaired"}
\description{
  Represents the reusults of computing the Newman Paired test statistic
  on one or more paired samples.
}
\section{Creating Objects}{
  In practice, users will use the \code{\link{pairedStat}} function to
  construct an object of the \code{NewmanPaired} class. Hand
  construction is strongly discouraged.
}
\section{Slots}{
  \describe{
    \item{\code{pairedMean}:}{A matrix of size N (number of features) by S (number
    of sample pairs). The mean expression of each feature in each paired
    sample. Also called "A" in the M-versus-A plots of the microarray era.}
    \item{\code{difference}:}{A matrix of size N (number of features) by S (number
    of sample pairs). The difference (perturbed - base) in expression of
    each feature in each paired sample. Also called "M" in the M-versus-A
    plots of the microarray era.}
    \item{\code{smoothSD}:}{A matrix of size N (number of features) by S (number
    of sample pairs). The results of fitting a loess smooth to the
    relationship between the \code{PairedMean} and the observed estimate
    of standard deviation (i.e., \code{abs(difference)/sqrt(2)}).}
    \item{\code{nuStatistics}:}{A matrix of size N (number of features) by S (number
    of sample pairs). The Newman paired statistics, nu.}
    \item{\code{pValues}:}{A matrix of size N (number of features) by S (number
    of sample pairs). Empirical p-values for the Newman statistics.}
  }
}
\section{Methods}{
  \describe{
    \item{x[i,j]}{Select a subset of features or sample pairs.}
    \item{dim(x)}{The dimension, N by S, of the object.}
    \item{plot(x, y, which = NULL, ask = NULL, high = 0.99, low = 0.01,
      ...)}{Plot the results of the analysis of one sample pair.}
    \item{hist(x, breaks=101, xlab="P-value", ...)}{Plot a histogram of the p-values for one sample-pair.}
  }
}
\references{
  Abrams ZB, Joglekar A, Gershkowitz GR, Sinicropi-yao S, Asiaee A,
  Carbone DP, Coombes KR. Personalized Transcriptomics: Selecting Drugs
  Based on Gene Expression Profiles. Preprint.
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com}
}
\seealso{
  \code{\link{pairedStat}}, \code{\link{bankStat}}
}
\examples{
showClass("NewmanPaired")
}
\keyword{classes}
\keyword{htest}
\keyword{multivariate}

\name{bankStat}
\alias{bankStat}
\alias{createBank}
\title{Newman Banked Statistic}
\description{
The Newman Banked Statistic is used to compare an individual sample to a cohort of similar samples.
}
\usage{
bankStat(bankObj, testSet, bankMatrix)
createBank(bankMatrix)
}
\arguments{
\item{bankObj}{Compressed representation of the cohort being compared to.}

\item{testSet}{Matrix containing data from one or more individual
  samples to be compared to the bank.}

\item{bankMatrix}{Data for the bank of "normal" or "untreated"
   or "baseline" control samples.}
}
\value{
A list containing two matrices: the \code{nu.statistics} and the \code{p.values}.
}
\examples{
data(GSE6631)
HN <- as.matrix(log2(1 + GSE6631))
bankMatrix <- HN[,seq(1, ncol(HN), 2)] # odd columns are normal
testSet <- HN[, seq(2, 6, 2)]   # evn columns are tumor
bs <- bankStat(testSet = testSet, bankMatrix = bankMatrix)
summary(bs$nu.statistics)
summary(bs$p.values)
}
